--
-- SellPriceTrigger
--
-- @Author: Tunis
-- @Date: 18.05.2022
-- @Version: 1.2.1.3
--
--
-- Changelog:
-- 1.2.1.0
-- Exclude owned prodution sellpoint
--
-- 1.2.0.2
-- Exclude hideFromPricesMenu sellpoint
--
-- 1.2.0.0
-- Prices per farm support in MP
-- Moved/added settings to ingame menu
--
-- 1.1.0.0
-- MP support
--
-- 1.0.1.0
-- Current price and trend (climbing or falling)
--

SellPriceTrigger = {}
SellPriceTrigger.moddir = g_currentModDirectory
SellPriceTrigger.triggers = {}
SellPriceTrigger.settings = {}
SellPriceTrigger.NOTIFICATION_TRIGGER = {0.1,0.1,0.5,1} -- Notification color
SellPriceTrigger.EVENTS_ID = {SEND_NUM_BITS=3,SAVE_TRIGGERS=1,RESET=2,RESET_GREAT=3,ADD_FARM=4,REMOVE_FARM=5,SHARED_SETTINGS=6,PREMISSIONS_SETTINGS=7}

SellPriceTrigger.debug = false
addModEventListener(SellPriceTrigger)

function SellPriceTrigger:loadMap()

	self.isDedi = g_currentMission.connectedToDedicatedServer
	if g_server ~= nil then
		if self.isDedi then
			self.isMP = true
		elseif g_currentMission.missionDynamicInfo ~= nil and g_currentMission.missionDynamicInfo.isMultiplayer then
			self.isMP = true
		else
			self.isMP = false
		end
	else
		self.isMP = true
	end

	BaseMission.unregisterActionEvents = Utils.appendedFunction(BaseMission.unregisterActionEvents, SellPriceTrigger.unregisterActionEvents)
	FSBaseMission.registerActionEvents = Utils.appendedFunction(FSBaseMission.registerActionEvents, SellPriceTrigger.registerActionEvents)
	
	FSBaseMission.onConnectionFinishedLoading = Utils.appendedFunction(FSBaseMission.onConnectionFinishedLoading, SellPriceTrigger.loadSettingsFromServer)
	-- Farms not exist in loadMap
	FSBaseMission.onFinishedLoading = Utils.appendedFunction(FSBaseMission.onFinishedLoading, SellPriceTrigger.onFinishedLoading)

	-- SaveSettings
	FSBaseMission.saveSavegame = Utils.appendedFunction(FSBaseMission.saveSavegame, SellPriceTrigger.saveCfg)

	-- Farms Menu
	InGameMenuMultiplayerFarmsFrame.deleteFarm = Utils.appendedFunction(InGameMenuMultiplayerFarmsFrame.deleteFarm, SellPriceTrigger.selectedFarm)
	InGameMenuMultiplayerFarmsFrame.onFarmCreated = Utils.appendedFunction(InGameMenuMultiplayerFarmsFrame.onFarmCreated, SellPriceTrigger.createFarm)
	InGameMenuMultiplayerFarmsFrame.onDeleteFarmYesNo = Utils.appendedFunction(InGameMenuMultiplayerFarmsFrame.onDeleteFarmYesNo, SellPriceTrigger.deleteFarm)

	-- Game Settings Menu
	if self.isMP then
		InGameMenuGameSettingsFrame.onFrameOpen = Utils.appendedFunction(InGameMenuGameSettingsFrame.onFrameOpen, SellPriceTrigger.initGameSettingsGui)
	end

	-- General Settings Menu
	InGameMenuGeneralSettingsFrame.onFrameOpen = Utils.appendedFunction(InGameMenuGeneralSettingsFrame.onFrameOpen, SellPriceTrigger.initGeneralSettingsGui)
	if g_server == nil then
		InGameMenuGeneralSettingsFrame.onFrameClose = Utils.appendedFunction(InGameMenuGeneralSettingsFrame.onFrameClose, SellPriceTrigger.saveCfg)
	end


	--Init
	self.updateTimer = 0.0
	self.updateRun = 3000
	self.updateEvery = 60000
	self.lastHour = 25
	self.showHelp = true
	self.soundLevel = 80
	self.selectedSound = 1

	self.settings.sharedTriggers = false
	self.settings.everyonePermission = false

	--Sound
	self.sounds = {}
	self.sounds[1] = createSample("TriggerSound1")
	loadSample(self.sounds[1], "data/sounds/ui/uiTransaction.wav", false)
	self.sounds[2] = createSample("TriggerSound2")
	loadSample(self.sounds[2], "data/sounds/ui/uiNotification.wav", false)
	self.sounds[3] = createSample("TriggerSound3")
	loadSample(self.sounds[3], "data/sounds/ui/uiCollectable.wav", false)
	self.sounds[4] = createSample("TriggerSound4")
	loadSample(self.sounds[4], "data/sounds/ui/uiSuccess.wav", false)

	--gui
	g_gui:loadProfiles(self.moddir .. "gui/guiProfiles.xml")
	g_gui:loadGui(self.moddir .. "gui/SellPriceTriggerGUI.xml", "SellPriceTriggerGUI", SellPriceTriggerGUI:new())

	-- mean calc
	g_messageCenter:subscribe(MessageType.HOUR_CHANGED, self.hourChanged, self)
end

function SellPriceTrigger:onFinishedLoading()
	SellPriceTrigger:loadCfg()

	SellPriceTrigger.sound = SellPriceTrigger.sounds[SellPriceTrigger.selectedSound]
end

function SellPriceTrigger:checkPrices(mean)

	mean = mean or false

	if self.settings.sharedTriggers then
		self:checkPricesShared(mean)
	else
		self:checkPricesPerFarm(mean)
	end
	
	self.updateTimer = 0
end

-- per farmTriggers
function SellPriceTrigger:checkPricesPerFarm(mean)

	local prices = {}

	for _,station in pairs(g_currentMission.storageSystem.unloadingStations) do

		if station.isSellingPoint ~= nil and station.isSellingPoint == true and station.hideFromPricesMenu == false then
			for fillType,isAccepted in pairs(station.acceptedFillTypes) do
				if isAccepted == true then

					local price = station:getEffectiveFillTypePrice(fillType)*1000
					local greatDemand = false
					if station.greatDemandFillType ~= nil and station.greatDemandFillType == fillType then
						greatDemand = true
					end

					for _,farm in pairs(g_farmManager.farms) do
						if farm.farmId > 0 and station.ownerFarmId ~= farm.farmId then

							if prices[fillType] == nil then
								prices[fillType] = {farms={}}
							end

							if prices[fillType].farms[farm.farmId] ~= nil then
								if price > prices[fillType].farms[farm.farmId].price then
									prices[fillType].farms[farm.farmId].price = price
									prices[fillType].farms[farm.farmId].stationName = station.uiName
									prices[fillType].farms[farm.farmId].greatDemand = greatDemand
									prices[fillType].farms[farm.farmId].priceTrend = station:getCurrentPricingTrend(fillType)
								end
								if price < prices[fillType].farms[farm.farmId].minPrice then
									prices[fillType].farms[farm.farmId].minPrice = price
								end
							else
								prices[fillType].farms[farm.farmId] = {price=0.0,priceTrend=0,stationName="",greatDemand=false,minPrice=0.0,meanTotal=0.0,meanCount=0}
								prices[fillType].farms[farm.farmId].price = price
								prices[fillType].farms[farm.farmId].minPrice = price
								prices[fillType].farms[farm.farmId].stationName = station.uiName
								prices[fillType].farms[farm.farmId].greatDemand = greatDemand
								prices[fillType].farms[farm.farmId].priceTrend = station:getCurrentPricingTrend(fillType)
							end

							if mean == true and price > 0 and greatDemand == false then -- exclude great demand from mean
								prices[fillType].farms[farm.farmId].meanTotal = prices[fillType].farms[farm.farmId].meanTotal + price
								prices[fillType].farms[farm.farmId].meanCount = prices[fillType].farms[farm.farmId].meanCount + 1
							end

						end
					end

				end
			end
		end
	end

	for fillType,trigger in pairs(self.triggers) do

		if prices[fillType] ~= nil then
			for farmId,farmTrigger in pairs(trigger.farms) do
				if prices[fillType].farms[farmId] ~= nil then

					if g_currentMission.player.farmId == farmId then
						if farmTrigger.triggerPrice > 0 and prices[fillType].farms[farmId].price > farmTrigger.triggerPrice then
							if farmTrigger.over == false or trigger.station ~= prices[fillType].farms[farmId].stationName then
								trigger.station = prices[fillType].farms[farmId].stationName
								local mess = ('%s %s @ %s'):format(g_fillTypeManager.indexToTitle[fillType], g_i18n:formatMoney(prices[fillType].farms[farmId].price), prices[fillType].farms[farmId].stationName)
								g_currentMission:addIngameNotification(self.NOTIFICATION_TRIGGER, mess, "SPTupdate")
								playSample(self.sound, 1, (self.soundLevel/100), 1, 0, 0)
								farmTrigger.over = true
							end
						else
							if farmTrigger.over == true then
								farmTrigger.over = false
							end
						end
						trigger.currentPrice = prices[fillType].farms[farmId].price
						trigger.currentTrend = prices[fillType].farms[farmId].priceTrend
					end

					if prices[fillType].farms[farmId].greatDemand == true then
						if prices[fillType].farms[farmId].price > farmTrigger.great then
							farmTrigger.great = prices[fillType].farms[farmId].price
						end	
					else
						if prices[fillType].farms[farmId].price > farmTrigger.highest then
							farmTrigger.highest = prices[fillType].farms[farmId].price
						end
					end

					if prices[fillType].farms[farmId].minPrice < farmTrigger.lowest or farmTrigger.lowest == 0 then
						farmTrigger.lowest = prices[fillType].farms[farmId].minPrice
					end

					if mean == true and prices[fillType].farms[farmId].meanTotal > 0 then
						farmTrigger.meanTotal = farmTrigger.meanTotal + (prices[fillType].farms[farmId].meanTotal / prices[fillType].farms[farmId].meanCount)
						farmTrigger.meanCount = farmTrigger.meanCount + 1
						farmTrigger.mean = farmTrigger.meanTotal / farmTrigger.meanCount
						--[[if farmTrigger.meanTotal > 10000 then
							farmTrigger.meanTotal = farmTrigger.mean
							farmTrigger.meanCount = 1
						end]]
					end
					
				end
			end

		end

	end
end

-- shared farmTriggers
function SellPriceTrigger:checkPricesShared(mean)

	local prices = {}

	for _,station in pairs(g_currentMission.storageSystem.unloadingStations) do

		if station.isSellingPoint ~= nil and station.isSellingPoint == true and station.hideFromPricesMenu == false then
			for fillType,isAccepted in pairs(station.acceptedFillTypes) do
				if isAccepted == true then

					local price = station:getEffectiveFillTypePrice(fillType)*1000
					local greatDemand = false
					if station.greatDemandFillType ~= nil and station.greatDemandFillType == fillType then
						greatDemand = true
					end

					for _,farm in pairs(g_farmManager.farms) do
						if farm.farmId > 0 and station.ownerFarmId ~= farm.farmId then

							if prices[fillType] == nil then
								prices[fillType] = {farms={},meanTotal=0.0,meanCount=0}
							end

							if prices[fillType].farms[farm.farmId] ~= nil then
								if price > prices[fillType].farms[farm.farmId].price then
									prices[fillType].farms[farm.farmId].price = price
									prices[fillType].farms[farm.farmId].stationName = station.uiName
									prices[fillType].farms[farm.farmId].greatDemand = greatDemand
									prices[fillType].farms[farm.farmId].priceTrend = station:getCurrentPricingTrend(fillType)
								end
								if price < prices[fillType].farms[farm.farmId].minPrice then
									prices[fillType].farms[farm.farmId].minPrice = price
								end
							else
								prices[fillType].farms[farm.farmId] = {price=0.0,priceTrend=0,stationName="",greatDemand=false,minPrice=0.0}
								prices[fillType].farms[farm.farmId].price = price
								prices[fillType].farms[farm.farmId].minPrice = price
								prices[fillType].farms[farm.farmId].stationName = station.uiName
								prices[fillType].farms[farm.farmId].greatDemand = greatDemand
								prices[fillType].farms[farm.farmId].priceTrend = station:getCurrentPricingTrend(fillType)
							end

						end
					end

					if mean == true and price > 0 and greatDemand == false then -- exclude great demand from mean
						prices[fillType].meanTotal = prices[fillType].meanTotal + price
						prices[fillType].meanCount = prices[fillType].meanCount + 1
					end

				end
			end
		end
	end

	for fillType,price in pairs(prices) do
		if self.triggers[fillType] ~= nil then
			local ftPrice = {price=0.0,priceTrend=0,greatDemand=false,minPrice=0.0} 
			for farmId,farmPrice in pairs(price.farms) do

				if g_currentMission.player.farmId == farmId then
					if self.triggers[fillType].farms[1].triggerPrice > 0 and farmPrice.price > self.triggers[fillType].farms[1].triggerPrice then
						if self.triggers[fillType].farms[1].over == false or self.triggers[fillType].station ~= farmPrice.stationName then
							self.triggers[fillType].station = farmPrice.stationName
							local mess = ('%s %s @ %s'):format(g_fillTypeManager.indexToTitle[fillType], g_i18n:formatMoney(farmPrice.price), farmPrice.stationName)
							g_currentMission:addIngameNotification(self.NOTIFICATION_TRIGGER, mess, "SPTupdate")
							playSample(self.sound, 1, (self.soundLevel/100), 1, 0, 0)
							self.triggers[fillType].farms[1].over = true
						end
					else
						if self.triggers[fillType].farms[1].over == true then
							self.triggers[fillType].farms[1].over = false
						end
					end
					self.triggers[fillType].currentPrice = farmPrice.price
					self.triggers[fillType].currentTrend = farmPrice.priceTrend
				end

				if farmPrice.price > ftPrice.price then
					ftPrice.price = farmPrice.price
					ftPrice.priceTrend = farmPrice.priceTrend
					ftPrice.greatDemand = farmPrice.greatDemand
				end

				if farmPrice.minPrice < ftPrice.minPrice or ftPrice.minPrice == 0 then
					ftPrice.minPrice = farmPrice.minPrice
				end

			end

			if ftPrice.greatDemand == true then
				if ftPrice.price > self.triggers[fillType].farms[1].great then
					self.triggers[fillType].farms[1].great = ftPrice.price
				end
			else
				if ftPrice.price > self.triggers[fillType].farms[1].highest then
					self.triggers[fillType].farms[1].highest = ftPrice.price
				end
			end

			if ftPrice.minPrice < self.triggers[fillType].farms[1].lowest or self.triggers[fillType].farms[1].lowest == 0 then
				self.triggers[fillType].farms[1].lowest = ftPrice.minPrice
			end

			if mean == true and prices[fillType].meanTotal > 0 then
				self.triggers[fillType].farms[1].meanTotal = self.triggers[fillType].farms[1].meanTotal + (prices[fillType].meanTotal / prices[fillType].meanCount)
				self.triggers[fillType].farms[1].meanCount = self.triggers[fillType].farms[1].meanCount + 1
				self.triggers[fillType].farms[1].mean = self.triggers[fillType].farms[1].meanTotal / self.triggers[fillType].farms[1].meanCount
			end
		end
	end
end

function SellPriceTrigger:meanPrice(index, farmId) -- index = fillType

	local prices = nil

	for _,station in pairs(g_currentMission.storageSystem.unloadingStations) do

		if station.isSellingPoint ~= nil and station.isSellingPoint == true and station.hideFromPricesMenu == false then
			if not self.settings.sharedTriggers and station.ownerFarmId == farmId then goto continue end
			for fillType,isAccepted in pairs(station.acceptedFillTypes) do
				if isAccepted == true and fillType == index then
				
					local price = station:getEffectiveFillTypePrice(fillType)*1000
					local greatDemand = false
					if station.greatDemandFillType ~= nil and station.greatDemandFillType == fillType then
						greatDemand = true
					end

					if prices == nil then
						prices = {meanTotal=0.0,meanCount=0}
					end

					if price > 0 and greatDemand == false then
							prices.meanTotal = prices.meanTotal + price
							prices.meanCount = prices.meanCount + 1
					end

				end
			end
		end
		::continue::
	end

	if self.triggers[index] ~= nil and prices ~= nil then
		if prices.meanTotal > 0 then
			self.triggers[index].farms[farmId].meanTotal = self.triggers[index].farms[farmId].meanTotal + (prices.meanTotal / prices.meanCount)
			self.triggers[index].farms[farmId].meanCount = self.triggers[index].farms[farmId].meanCount + 1
			self.triggers[index].farms[farmId].mean = self.triggers[index].farms[farmId].meanTotal / self.triggers[index].farms[farmId].meanCount
		end
	end

end

function SellPriceTrigger:fillTypes()
	for _,filltypes in pairs(g_fillTypeManager.fillTypes) do
		if filltypes.showOnPriceTable ~= nil and filltypes.showOnPriceTable == true then
			if self.debug then
				print("---- "..tostring(filltypes.index).." "..tostring(filltypes.name).." "..tostring(filltypes.title))
			end
			self.triggers[filltypes.index] = {index=0,station="",currentPrice=0.0,currentTrend=0,farms={}}
			self.triggers[filltypes.index].index = filltypes.index
			if self.settings.sharedTriggers then
				if self.debug then
					print("--- FarmId 1 Shared")
				end
				self.triggers[filltypes.index].farms[1] = {farmId=1,triggerPrice=0,lowest=0.0,mean=0.0,highest=0.0,great=0.0,over=false,meanTotal=0.0,meanCount=0}
			else
				for _,farm in pairs(g_farmManager.farms) do
					if farm.farmId > 0 then
						if self.debug then
								print("--- FarmId "..tostring(farm.farmId))
						end
						self.triggers[filltypes.index].farms[farm.farmId] = {farmId=0,triggerPrice=0,lowest=0.0,mean=0.0,highest=0.0,great=0.0,over=false,meanTotal=0.0,meanCount=0}
						self.triggers[filltypes.index].farms[farm.farmId].farmId = farm.farmId
					end
				end
			end
		end
	end
end

-- Farm Manage
function SellPriceTrigger:addFarmTriggers(farmId)
	if SellPriceTrigger.debug then
		print("---- Added farm: " .. tostring(farmId))
	end
	for _,trigger in pairs(SellPriceTrigger.triggers) do
		trigger.farms[farmId] = {farmId=0,triggerPrice=0,lowest=0.0,mean=0.0,highest=0.0,great=0.0,over=false,meanTotal=0.0,meanCount=0}
		trigger.farms[farmId].farmId = farmId
	end
end

function SellPriceTrigger:removeFarmTriggers(farmId)
	if SellPriceTrigger.debug then
		print("---- Deleted farm: " .. tostring(farmId))
	end
	for _,trigger in pairs(SellPriceTrigger.triggers) do
		trigger.farms[farmId] = nil
	end
end

function SellPriceTrigger:createFarm(farmId)
	if SellPriceTrigger.settings.sharedTriggers then
		if #g_farmManager.farms == 2 then
			farmId = 1
		else
			return
		end
	end
	
	SellPriceTrigger:onChangeEvent(SellPriceTrigger.EVENTS_ID.ADD_FARM, farmId, true)
end

function SellPriceTrigger:selectedFarm(farmId)
	self.sptFarmId = farmId
end

function SellPriceTrigger:deleteFarm(delete)
	if delete then
		local farmId = self.sptFarmId
		if SellPriceTrigger.settings.sharedTriggers then
			if #g_farmManager.farms == 1 then
				farmId = 1
			else
				return
			end
		end
		SellPriceTrigger:onChangeEvent(SellPriceTrigger.EVENTS_ID.REMOVE_FARM, farmId, false)
	end
end

-- Load
function SellPriceTrigger:loadCfg()
	if g_server == nil then
		-- load client settings
		if getUserProfileAppPath() ~= nil then
			local file = getUserProfileAppPath() .. "modSettings/sellpricetrigger.xml"
			if SellPriceTrigger.debug then
				print("---Load file "..tostring(file))
			end
			if fileExists(file) then
				local XML = loadXMLFile("sellPriceTrigger_XML", file, "sellPriceTriggers")
				if self.debug then
					print("---Load Client XML")
				end

				if getXMLBool(XML, "sellPriceTriggers#showhelp") ~= nil then self.showHelp = getXMLBool(XML, "sellPriceTriggers#showhelp") end
				if getXMLBool(XML, "sellPriceTriggers#premission") ~= nil then self.settings.everyonePermission = getXMLBool(XML, "sellPriceTriggers#premission") end
				if getXMLBool(XML, "sellPriceTriggers#shared") ~= nil then self.settings.sharedTriggers = getXMLBool(XML, "sellPriceTriggers#shared") end
				
				if getXMLFloat(XML, "sellPriceTriggers#R") ~= nil and getXMLFloat(XML, "sellPriceTriggers#G") ~= nil and getXMLFloat(XML, "sellPriceTriggers#B") ~= nil then
					local R = getXMLFloat(XML, "sellPriceTriggers#R")
					local G = getXMLFloat(XML, "sellPriceTriggers#G")
					local B = getXMLFloat(XML, "sellPriceTriggers#B")
					self.NOTIFICATION_TRIGGER = {R,G,B,1}
					if self.debug then
						print("---Load R "..tostring(R).." G "..tostring(G).." B "..tostring(B))
					end
				end
				
				if getXMLInt(XML, "sellPriceTriggers#soundlevel") ~= nil then self.soundLevel = getXMLInt(XML, "sellPriceTriggers#soundlevel") end
				if getXMLInt(XML, "sellPriceTriggers#sound") ~= nil then self.selectedSound = getXMLInt(XML, "sellPriceTriggers#sound") end
			end
		end
	else
		if g_currentMission.missionInfo.savegameDirectory ~= nil then
			local file = g_currentMission.missionInfo.savegameDirectory .. "/sellpricetrigger.xml"
			if fileExists(file) then
				local XML = loadXMLFile("sellPriceTrigger_XML", file, "sellPriceTriggers")
				if self.debug then
					print("---Load XML")
				end

				if getXMLBool(XML, "sellPriceTriggers#shared") == nil then
					if self.debug then
						print("---Old cfg")
					end
					self:fillTypes()
					self:loadOldCfg(XML)
					return
				end

				if getXMLBool(XML, "sellPriceTriggers#showhelp") ~= nil then self.showHelp = getXMLBool(XML, "sellPriceTriggers#showhelp") end
				if getXMLBool(XML, "sellPriceTriggers#premission") ~= nil then self.settings.everyonePermission = getXMLBool(XML, "sellPriceTriggers#premission") end
				if getXMLBool(XML, "sellPriceTriggers#shared") ~= nil then self.settings.sharedTriggers = getXMLBool(XML, "sellPriceTriggers#shared") end

				if getXMLFloat(XML, "sellPriceTriggers#R") ~= nil and getXMLFloat(XML, "sellPriceTriggers#G") ~= nil and getXMLFloat(XML, "sellPriceTriggers#B") ~= nil then
					local R = getXMLFloat(XML, "sellPriceTriggers#R")
					local G = getXMLFloat(XML, "sellPriceTriggers#G")
					local B = getXMLFloat(XML, "sellPriceTriggers#B")
					self.NOTIFICATION_TRIGGER = {R,G,B,1}
					if self.debug then
						print("---Load R "..tostring(R).." G "..tostring(G).." B "..tostring(B))
					end
				end

				if getXMLInt(XML, "sellPriceTriggers#soundlevel") ~= nil then self.soundLevel = getXMLInt(XML, "sellPriceTriggers#soundlevel") end
				if getXMLInt(XML, "sellPriceTriggers#sound") ~= nil then self.selectedSound = getXMLInt(XML, "sellPriceTriggers#sound") end

				self:fillTypes()

				for x=0,1000 do

					local groupFilltypeTag = ("sellPriceTriggers.fillTypes.fillType(%d)"):format(x)
					local name = getXMLString(XML, groupFilltypeTag.. "#name")
					if self.debug then
						print("---XML "..tostring(x).." name "..tostring(name))
					end

					if name == nil then
						break
					end

					if self.triggers[g_fillTypeManager.nameToIndex[name]] ~= nil then
						if self.debug then
							print("---XML found name "..tostring(name))
						end

						for y=0,100 do
							local groupFarmTag = groupFilltypeTag..(".farms.farm(%d)"):format(y)
							local farmId = getXMLInt(XML, groupFarmTag.. "#id")
							if self.debug then
								print("---XML  "..tostring(y).." farmId "..tostring(farmId))
							end
							
							if farmId == nil or self.triggers[g_fillTypeManager.nameToIndex[name]].farms[farmId] == nil then
								break
							end
							if getXMLInt(XML, groupFarmTag.. "#price") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[farmId].triggerPrice = getXMLInt(XML, groupFarmTag.. "#price") end
							if getXMLFloat(XML, groupFarmTag.. "#highest") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[farmId].highest = getXMLFloat(XML, groupFarmTag.. "#highest") end
							if getXMLFloat(XML, groupFarmTag.. "#great") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[farmId].great = getXMLFloat(XML, groupFarmTag.. "#great") end
							if getXMLFloat(XML, groupFarmTag.. "#lowest") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[farmId].lowest = getXMLFloat(XML, groupFarmTag.. "#lowest") end
							if getXMLFloat(XML, groupFarmTag.. "#mean") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[farmId].mean = getXMLFloat(XML, groupFarmTag.. "#mean") end
							if getXMLFloat(XML, groupFarmTag.. "#meantotal") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[farmId].meanTotal = getXMLFloat(XML, groupFarmTag.. "#meantotal") end
							if getXMLInt(XML, groupFarmTag.. "#meancount") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[farmId].meanCount = getXMLInt(XML, groupFarmTag.. "#meancount") end
						end
					end
				end
			end
		end
		if #self.triggers < 1 then
			self:fillTypes()
		end
	end
	if SellPriceTrigger.debug then
		print("-Load getIsServer: "..tostring(g_currentMission:getIsServer()).." getIsClient: "..tostring(g_currentMission:getIsClient()).." Connected to Dedi: "..tostring(g_currentMission.connectedToDedicatedServer))
	end
end

function SellPriceTrigger:loadOldCfg(XML)
	if getXMLBool(XML, "sellPriceTriggers#showhelp") ~= nil then self.showHelp = getXMLBool(XML, "sellPriceTriggers#showhelp") end

	if getXMLFloat(XML, "sellPriceTriggers#R") ~= nil and getXMLFloat(XML, "sellPriceTriggers#G") ~= nil and getXMLFloat(XML, "sellPriceTriggers#B") ~= nil then
		local R = getXMLFloat(XML, "sellPriceTriggers#R")
		local G = getXMLFloat(XML, "sellPriceTriggers#G")
		local B = getXMLFloat(XML, "sellPriceTriggers#B")
		self.NOTIFICATION_TRIGGER = {R,G,B,1}
		if self.debug then
			print("---Load R "..tostring(R).." G "..tostring(G).." B "..tostring(B))
		end
	end

	--if getXMLFloat(XML, "sellPriceTriggers#soundlevel") ~= nil then self.soundLevel = getXMLFloat(XML, "sellPriceTriggers#soundlevel") end

	for x=0,1000 do

		local groupNameTag = ("sellPriceTriggers.fillTypes.fillType(%d)"):format(x)
		local name = getXMLString(XML, groupNameTag.. "#name")
		if self.debug then
			print("---XML "..tostring(x).." name "..tostring(name))
		end

		if name == nil then
			break
		end

		if self.triggers[g_fillTypeManager.nameToIndex[name]] ~= nil then
			if self.debug then
				print("---XML found name "..tostring(name))
			end
			if getXMLInt(XML, groupNameTag.. "#price") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[1].triggerPrice = getXMLInt(XML, groupNameTag.. "#price") end
			if getXMLFloat(XML, groupNameTag.. "#highest") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[1].highest = getXMLFloat(XML, groupNameTag.. "#highest") end
			if getXMLFloat(XML, groupNameTag.. "#great") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[1].great = getXMLFloat(XML, groupNameTag.. "#great") end
			if getXMLFloat(XML, groupNameTag.. "#lowest") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[1].lowest = getXMLFloat(XML, groupNameTag.. "#lowest") end
			if getXMLFloat(XML, groupNameTag.. "#mean") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[1].mean = getXMLFloat(XML, groupNameTag.. "#mean") end
			if getXMLFloat(XML, groupNameTag.. "#meantotal") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[1].meanTotal = getXMLFloat(XML, groupNameTag.. "#meantotal") end
			if getXMLInt(XML, groupNameTag.. "#meancount") ~= nil then self.triggers[g_fillTypeManager.nameToIndex[name]].farms[1].meanCount = getXMLInt(XML, groupNameTag.. "#meancount") end
		end

	end
end

function SellPriceTrigger:saveCfg()
	if g_server == nil then 
		-- save client settings
		if getUserProfileAppPath() ~= nil then
			local file = getUserProfileAppPath() .. "modSettings/sellpricetrigger.xml"
			if SellPriceTrigger.debug then
				print("---Save file "..tostring(file))
			end
			local XML = createXMLFile("sellPriceTrigger_XML", file, "sellPriceTriggers")
			if SellPriceTrigger.debug then
				print("---Save XML")
			end

			setXMLBool(XML, "sellPriceTriggers#showhelp", SellPriceTrigger.showHelp)
			setXMLBool(XML, "sellPriceTriggers#premission", SellPriceTrigger.settings.everyonePermission)
			setXMLBool(XML, "sellPriceTriggers#shared", SellPriceTrigger.settings.sharedTriggers)
			setXMLFloat(XML, "sellPriceTriggers#R", SellPriceTrigger.NOTIFICATION_TRIGGER[1])
			setXMLFloat(XML, "sellPriceTriggers#G", SellPriceTrigger.NOTIFICATION_TRIGGER[2])
			setXMLFloat(XML, "sellPriceTriggers#B", SellPriceTrigger.NOTIFICATION_TRIGGER[3])
			setXMLFloat(XML, "sellPriceTriggers#soundlevel", SellPriceTrigger.soundLevel)
			setXMLInt(XML, "sellPriceTriggers#sound", SellPriceTrigger.selectedSound)

			saveXMLFile(XML)
		end
	else	
		if g_currentMission.missionInfo.savegameDirectory ~= nil then
			local file = g_currentMission.missionInfo.savegameDirectory .. "/sellpricetrigger.xml"
			local XML = createXMLFile("sellPriceTrigger_XML", file, "sellPriceTriggers")
			local x = 0
			if SellPriceTrigger.debug then
				print("---Save XML")
			end

			setXMLBool(XML, "sellPriceTriggers#showhelp", SellPriceTrigger.showHelp)
			setXMLBool(XML, "sellPriceTriggers#premission", SellPriceTrigger.settings.everyonePermission)
			setXMLBool(XML, "sellPriceTriggers#shared", SellPriceTrigger.settings.sharedTriggers)
			setXMLFloat(XML, "sellPriceTriggers#R", SellPriceTrigger.NOTIFICATION_TRIGGER[1])
			setXMLFloat(XML, "sellPriceTriggers#G", SellPriceTrigger.NOTIFICATION_TRIGGER[2])
			setXMLFloat(XML, "sellPriceTriggers#B", SellPriceTrigger.NOTIFICATION_TRIGGER[3])
			setXMLInt(XML, "sellPriceTriggers#soundlevel", SellPriceTrigger.soundLevel)
			setXMLInt(XML, "sellPriceTriggers#sound", SellPriceTrigger.selectedSound)
			for fillType,trigger in pairs(SellPriceTrigger.triggers) do

				local groupFilltypeTag = ("sellPriceTriggers.fillTypes.fillType(%d)"):format(x)
				setXMLString(XML, groupFilltypeTag.. "#name", g_fillTypeManager.indexToName[trigger.index])

				local y = 0
				for farmId,farmTrigger in pairs(trigger.farms) do
					--if farmId > 0 then
						local groupFarmTag = groupFilltypeTag..(".farms.farm(%d)"):format(y)
						setXMLInt(XML, groupFarmTag.. "#id", farmTrigger.farmId)
						setXMLInt(XML, groupFarmTag.. "#price", farmTrigger.triggerPrice)
						setXMLFloat(XML, groupFarmTag.. "#highest", farmTrigger.highest)
						setXMLFloat(XML, groupFarmTag.. "#great", farmTrigger.great)
						setXMLFloat(XML, groupFarmTag.. "#lowest", farmTrigger.lowest)
						setXMLFloat(XML, groupFarmTag.. "#mean", farmTrigger.mean)
						setXMLFloat(XML, groupFarmTag.. "#meantotal", farmTrigger.meanTotal)
						setXMLInt(XML, groupFarmTag.. "#meancount", farmTrigger.meanCount)
						y=y+1
					--end
				end
				x=x+1

			end

			saveXMLFile(XML)
		end
	end
	if SellPriceTrigger.debug then
		print("-Save getIsServer: "..tostring(g_currentMission:getIsServer()).." getIsClient: "..tostring(g_currentMission:getIsClient()).." Connected to Dedi: "..tostring(g_currentMission.connectedToDedicatedServer))
	end
end

function SellPriceTrigger:update(dt)
	self.updateTimer = self.updateTimer + dt * g_currentMission.missionInfo.timeScale
	if self.updateTimer >= self.updateRun then
		self.updateRun = self.updateEvery
		self:checkPrices()
	end
end

function SellPriceTrigger:hourChanged()
	if SellPriceTrigger.debug then
		print("---hourChanged "..tostring(g_currentMission.environment.currentHour))
	end
	--if SellPriceTrigger.lastHour ~= g_currentMission.environment.currentHour then
		SellPriceTrigger.lastHour = g_currentMission.environment.currentHour
		SellPriceTrigger:checkPrices(true)
	--end
end

function SellPriceTrigger:mouseEvent(posX, posY, isDown, isUp, button) end
function SellPriceTrigger:keyEvent(unicode, sym, modifier, isDown) end
function SellPriceTrigger:draw() end
function SellPriceTrigger:deleteMap() end

function SellPriceTrigger:delete()
	g_messageCenter:unsubscribeAll(self)
end

function SellPriceTrigger:registerActionEvents()
	if g_currentMission:getIsClient() then
		local _, eventId = g_gui.inputManager:registerActionEvent(InputAction.SPT_SHOW_UI, self, SellPriceTrigger.actionEventShowUI, false, true, false, true, nil, nil, true)
		SellPriceTrigger.eventId = eventId
		self.inputManager:setActionEventTextVisibility(eventId, SellPriceTrigger.showHelp)
	end
end

function SellPriceTrigger:unregisterActionEvents()
	g_gui.inputManager:removeActionEventsByTarget(self)
end

function SellPriceTrigger:updateActionEvents()
	--Update Show Help
	g_gui.inputManager:setActionEventTextVisibility(SellPriceTrigger.eventId, SellPriceTrigger.showHelp)
end

function SellPriceTrigger:actionEventShowUI()
	if g_gui:getIsGuiVisible() then
		return
	end
	if g_currentMission.player.farmId == 0 then
		g_currentMission:showBlinkingWarning(g_i18n:getText("SPT_warning_NoFarm"), 3000)
		return
	end
	g_gui:showGui("SellPriceTriggerGUI")
end


-- Load from server
function SellPriceTrigger:loadSettingsFromServer()
	g_client:getServerConnection():sendEvent(LoadSellPriceTriggerEvent.new())
end

function SellPriceTrigger:onChangeEvent(id, farmId, data)
	g_client:getServerConnection():sendEvent(ChangeSellPriceTriggerEvent.new(id, farmId, data))
end


-- Menus
function SellPriceTrigger:initGameSettingsGui()
	if self.sptTriggers == nil then
		--Create Triggers Element
		self.sptTriggers = self.checkTraffic:clone()
		self.sptTriggers.target = false
		self.sptTriggers.id = "sptTriggers"
		self.sptTriggers.onClickCallback = SellPriceTrigger.onSettingsStateChanged
		self.sptTriggers.texts[1] = g_i18n:getText('SPT_gui_SettingsTriggersStateOne')
		self.sptTriggers.texts[2] = g_i18n:getText('SPT_gui_SettingsTriggersStateTwo')

		local settingTitle = self.sptTriggers.elements[4]
		local toolTip = self.sptTriggers.elements[6]

		settingTitle:setText(g_i18n:getText('SPT_gui_SettingsTriggers'))
		toolTip:setText(g_i18n:getText('SPT_gui_SettingsTriggersToolTip'))

		--- Create Permission Element
		self.sptPermissions =  self.sptTriggers:clone()
		self.sptPermissions.id = "sptPermissions"
		self.sptPermissions.texts[1] = g_i18n:getText('SPT_gui_SettingsPermissionsStateOne')
		self.sptPermissions.texts[2] = g_i18n:getText('SPT_gui_SettingsPermissionsStateTwo')

		settingTitle = self.sptPermissions.elements[4]
		toolTip = self.sptPermissions.elements[6]

		settingTitle:setText(g_i18n:getText('SPT_gui_SettingsPermissions'))
		toolTip:setText(g_i18n:getText('SPT_gui_SettingsPermissionsToolTip'))

		--- Create Menu Elements
		local title = TextElement.new()
		title:applyProfile("settingsMenuSubtitle", true)
		title:setText(g_i18n:getText('SPT_gui_Title'))

		self.boxLayout:addElement(title)
		self.boxLayout:addElement(self.sptTriggers)
		self.boxLayout:addElement(self.sptPermissions)
		self.boxLayout:invalidateLayout()
	end
	
	self.sptTriggers:setIsChecked(SellPriceTrigger.settings.sharedTriggers)
	self.sptPermissions:setIsChecked(SellPriceTrigger.settings.everyonePermission)
end

function SellPriceTrigger:initGeneralSettingsGui()
	if self.sptSound == nil then
		--- Create Sound Element
		self.sptSound = self.multiMoneyUnit:clone()
		self.sptSound.target = false
		self.sptSound.id = "sptSound"
		self.sptSound.onClickCallback = SellPriceTrigger.onSettingsStateChanged
		self.sptSound.texts[1] = "1"
		self.sptSound.texts[2] = "2"
		self.sptSound.texts[3] = "3"
		self.sptSound.texts[4] = "4"

		settingTitle = self.sptSound.elements[4]
		toolTip = self.sptSound.elements[6]

		settingTitle:setText(g_i18n:getText('SPT_gui_SettingsSound'))
		toolTip:setText(g_i18n:getText('SPT_gui_SettingsSoundToolTip'))

		self.sptSound:setState(SellPriceTrigger.selectedSound)

		--- Create SoundLevel Element
		self.sptSoundLevel = self.multiMasterVolume:clone()
		self.sptSoundLevel.target = false
		self.sptSoundLevel.id = "sptSoundLevel"
		self.sptSoundLevel.onClickCallback = SellPriceTrigger.onSettingsStateChanged

		settingTitle = self.sptSoundLevel.elements[4]
		toolTip = self.sptSoundLevel.elements[6]

		settingTitle:setText(g_i18n:getText('SPT_gui_SettingsSoundLevel'))
		toolTip:setText(g_i18n:getText('SPT_gui_SettingsSoundLevelToolTip'))

		local state = 5
		if(SellPriceTrigger.soundLevel ~= nil) then
			state = SellPriceTrigger.soundLevel / 10 + 1
		end
		self.sptSoundLevel:setState(state)

		-- Create showhelp Element
		self.sptShowHelp = self.checkAutoHelp:clone()
		self.sptShowHelp.target = false
		self.sptShowHelp.id = "sptShowHelp"
		self.sptShowHelp.onClickCallback = SellPriceTrigger.onSettingsStateChanged

		settingTitle = self.sptShowHelp.elements[4]
		toolTip = self.sptShowHelp.elements[6]

		settingTitle:setText(g_i18n:getText('SPT_gui_SettingsShowHelp'))
		toolTip:setText(g_i18n:getText('SPT_gui_SettingsShowHelpToolTip'))

		self.sptShowHelp:setIsChecked(SellPriceTrigger.showHelp)
		
		--- Create Menu Elements
		local title = TextElement.new()
		title:applyProfile("settingsMenuSubtitle", true)
		title:setText(g_i18n:getText('SPT_gui_Title'))

		self.boxLayout:addElement(title)
		self.boxLayout:addElement(self.sptSound)
		self.boxLayout:addElement(self.sptSoundLevel)
		self.boxLayout:addElement(self.sptShowHelp)
		self.boxLayout:invalidateLayout()
	end
end

function SellPriceTrigger:onSettingsStateChanged(state, element, isChangedUp)
	if element.id == "sptTriggers" then
		g_gui:showYesNoDialog(
			{
				text = g_i18n:getText("SPT_gui_SettingsTriggersDialogText"),
				title = g_i18n:getText("SPT_gui_SettingsTriggersDialogTitle"),
				callback = SellPriceTrigger.onClickTriggersDialogCallback,
				target = element
			}
		)
	end

	if element.id == "sptPermissions" then
		if SellPriceTrigger.debug then
			print("Change Permissions Value: ".. tostring(state) .. " " .. tostring(element:getIsChecked()))
		end
		SellPriceTrigger:onChangeEvent(SellPriceTrigger.EVENTS_ID.PREMISSIONS_SETTINGS, 0, element:getIsChecked())
	end

	if element.id == "sptSound" then
		stopSample(SellPriceTrigger.sound,0,0)
		SellPriceTrigger.selectedSound = state
		SellPriceTrigger.sound = SellPriceTrigger.sounds[SellPriceTrigger.selectedSound]
		playSample(SellPriceTrigger.sound, 1, (SellPriceTrigger.soundLevel/100), 1, 0, 0)
	end

	if element.id == "sptSoundLevel" then
		stopSample(SellPriceTrigger.sound,0,0)
		SellPriceTrigger.soundLevel = (state - 1) * 10
		playSample(SellPriceTrigger.sound, 1, (SellPriceTrigger.soundLevel/100), 1, 0, 0)
	end
	
	if element.id == "sptShowHelp" then
		if SellPriceTrigger.debug then
			print("ShowHelp Value: ".. tostring(state) .. " " .. tostring(element:getIsChecked()))
		end
		SellPriceTrigger.showHelp = element:getIsChecked()
		SellPriceTrigger:updateActionEvents()
	end
end

function SellPriceTrigger:onClickTriggersDialogCallback(isYes)
	if isYes then
		if SellPriceTrigger.debug then
			print("Change Triggers Value: ".. tostring(self.state) .. " " .. tostring(self:getIsChecked()))
		end
		SellPriceTrigger:onChangeEvent(SellPriceTrigger.EVENTS_ID.SHARED_SETTINGS, 0, self:getIsChecked())
	else
		self:setIsChecked(SellPriceTrigger.settings.sharedTriggers)
	end
end
