--
-- ChangeSellPriceTriggerEvent
--
-- @Author: Tunis
-- @Date: 18.05.2022
-- @Version: 1.2.0.2
--
--
-- Changelog:
-- 1.2.0.0
-- Prices per farm support in MP
--

ChangeSellPriceTriggerEvent = {}
ChangeSellPriceTriggerEvent_mt = Class(ChangeSellPriceTriggerEvent, Event)
InitEventClass(ChangeSellPriceTriggerEvent, "ChangeSellPriceTriggerEvent")

function ChangeSellPriceTriggerEvent.emptyNew()
	local self = Event.new(ChangeSellPriceTriggerEvent_mt)
	return self
end

function ChangeSellPriceTriggerEvent.new(id, farmId, data)
	local self = ChangeSellPriceTriggerEvent.emptyNew()
	self.id = id
	self.farmId = farmId
	self.data = data
	return self
end

function ChangeSellPriceTriggerEvent:readStream(streamId, connection)
	if SellPriceTrigger.debug then
		print("ChangeSellPriceTriggerEvent:readStream")
	end
	self.id = streamReadUIntN(streamId, SellPriceTrigger.EVENTS_ID.SEND_NUM_BITS)
	self.farmId = streamReadUIntN(streamId, FarmManager.FARM_ID_SEND_NUM_BITS)

	if self.id == SellPriceTrigger.EVENTS_ID.SAVE_TRIGGERS then
		self.data = {}
		local numTriggers = streamReadUInt8(streamId)
		for i = 1, numTriggers do
			local trigger = {}
			trigger.index = streamReadUInt8(streamId)
			trigger.price = streamReadUInt16(streamId)
			if SellPriceTrigger.debug then print("readStream Triggers:" .. tostring(trigger.index) .. " Farm Id: " .. tostring(self.farmId)) end
			self.data[trigger.index] = trigger
		end
	end

	if self.id == SellPriceTrigger.EVENTS_ID.RESET then
		self.data = {}
		self.data.index = streamReadUInt8(streamId)
		local farm = {}
		farm.lowest = streamReadFloat32(streamId)
		farm.mean = streamReadFloat32(streamId)
		farm.highest = streamReadFloat32(streamId)
		farm.meanTotal = streamReadFloat32(streamId)
		farm.meanCount = streamReadUInt16(streamId)
		self.data.farms = {}
		self.data.farms[self.farmId] = farm
		if SellPriceTrigger.debug then print("readStream Reset:" .. tostring(self.data.index) .. " Farm Id: " .. tostring(self.farmId)) end
	end

	if self.id == SellPriceTrigger.EVENTS_ID.RESET_GREAT then
		self.data = {}
		self.data.index = streamReadUInt8(streamId)
		self.data.farms = {}
		self.data.farms[self.farmId] = {}
		self.data.farms[self.farmId].great = streamReadFloat32(streamId)
		if SellPriceTrigger.debug then print("readStream Reset great:" .. tostring(self.data.index) .. " Farm Id: " .. tostring(self.farmId)) end
	end

	if self.id == SellPriceTrigger.EVENTS_ID.SHARED_SETTINGS then
		self.data = streamReadBool(streamId)
		if SellPriceTrigger.debug then print("readStream sharedTriggers: " .. tostring(self.data)) end
	end

	if self.id == SellPriceTrigger.EVENTS_ID.PREMISSIONS_SETTINGS then
		self.data = streamReadBool(streamId)
		if SellPriceTrigger.debug then print("readStream everyonePermissions: " .. tostring(self.data)) end
	end

	self:run(connection)
end

function ChangeSellPriceTriggerEvent:writeStream(streamId, connection)

	if SellPriceTrigger.debug then
		print("ChangeSellPriceTriggerEvent:writeStream")
	end

	streamWriteUIntN(streamId, self.id, SellPriceTrigger.EVENTS_ID.SEND_NUM_BITS)
	streamWriteUIntN(streamId, self.farmId, FarmManager.FARM_ID_SEND_NUM_BITS)

	if self.id == SellPriceTrigger.EVENTS_ID.SAVE_TRIGGERS then
		count = 0
		for _ in pairs(self.data) do count = count + 1 end
		streamWriteUInt8(streamId, count)
		for _,v in pairs(self.data) do
			if SellPriceTrigger.debug then print("writeStream Triggers: " .. tostring(v.index)  .. " Farm Id: " .. tostring(self.farmId)) end
			streamWriteUInt8(streamId, v.index)
			streamWriteUInt16(streamId, v.price)
		end
	end

	if self.id == SellPriceTrigger.EVENTS_ID.RESET then
		if SellPriceTrigger.debug then print("writeStream Reset: " .. tostring(self.data.index)  .. " Farm Id: " .. tostring(self.farmId)) end
		streamWriteUInt8(streamId, self.data.index)
		streamWriteFloat32(streamId, self.data.farms[self.farmId].lowest)
		streamWriteFloat32(streamId, self.data.farms[self.farmId].mean)
		streamWriteFloat32(streamId, self.data.farms[self.farmId].highest)
		streamWriteFloat32(streamId, self.data.farms[self.farmId].meanTotal)
		streamWriteUInt16(streamId, self.data.farms[self.farmId].meanCount)
	end

	if self.id == SellPriceTrigger.EVENTS_ID.RESET_GREAT then
		if SellPriceTrigger.debug then print("writeStream Reset great: " .. tostring(self.data.index)  .. " Farm Id: " .. tostring(self.farmId)) end
		streamWriteUInt8(streamId, self.data.index)
		streamWriteFloat32(streamId, self.data.farms[self.farmId].great)
	end

	if self.id == SellPriceTrigger.EVENTS_ID.SHARED_SETTINGS then
		if SellPriceTrigger.debug then print("writeStream sharedTriggers: " .. tostring(self.data)) end
		streamWriteBool(streamId, self.data)
	end

	if self.id == SellPriceTrigger.EVENTS_ID.PREMISSIONS_SETTINGS then
		if SellPriceTrigger.debug then print("writeStream everyonePermissions: " .. tostring(self.data)) end
		streamWriteBool(streamId, self.data)
	end

end

function ChangeSellPriceTriggerEvent:run(connection)
	if(self.id ~= nil) then
		if SellPriceTrigger.debug then print("Id not nil") end
		if SellPriceTrigger.debug then print("ID: " .. tostring(self.id)) end

		if self.id == SellPriceTrigger.EVENTS_ID.SAVE_TRIGGERS then
			for _,trigger in pairs(self.data) do
				if SellPriceTrigger.debug then print("Save Triggers:" .. tostring(trigger.index) .. " Farm Id: " .. tostring(self.farmId)) end
				SellPriceTrigger.triggers[trigger.index].farms[self.farmId].triggerPrice = trigger.price
				SellPriceTrigger.triggers[trigger.index].farms[self.farmId].over = false
			end
			SellPriceTrigger:checkPrices()
		end

		if self.id == SellPriceTrigger.EVENTS_ID.RESET then
			if SellPriceTrigger.debug then print("Reset: " .. tostring(self.data.index) .. " Farm Id: " .. tostring(self.farmId)) end
			SellPriceTrigger.triggers[self.data.index].farms[self.farmId].lowest = self.data.farms[self.farmId].lowest
			SellPriceTrigger.triggers[self.data.index].farms[self.farmId].mean = self.data.farms[self.farmId].mean
			SellPriceTrigger.triggers[self.data.index].farms[self.farmId].highest = self.data.farms[self.farmId].highest
			SellPriceTrigger.triggers[self.data.index].farms[self.farmId].meanTotal = self.data.farms[self.farmId].meanTotal
			SellPriceTrigger.triggers[self.data.index].farms[self.farmId].meanCount = self.data.farms[self.farmId].meanCount
		end

		if self.id == SellPriceTrigger.EVENTS_ID.RESET_GREAT then
			if SellPriceTrigger.debug then print("Reset great: " .. tostring(self.data.index) .. " Farm Id: " .. tostring(self.farmId)) end
			SellPriceTrigger.triggers[self.data.index].farms[self.farmId].great = self.data.farms[self.farmId].great
		end

		if self.id == SellPriceTrigger.EVENTS_ID.ADD_FARM then
			if SellPriceTrigger.debug then print("Add farm: " .. tostring(self.farmId)) end
			SellPriceTrigger:addFarmTriggers(self.farmId)
		end

		if self.id == SellPriceTrigger.EVENTS_ID.REMOVE_FARM then
			if SellPriceTrigger.debug then print("Remove farm: " .. tostring(self.farmId)) end
			SellPriceTrigger:removeFarmTriggers(self.farmId)
		end

		if self.id == SellPriceTrigger.EVENTS_ID.SHARED_SETTINGS then
			if SellPriceTrigger.debug then print("Change sharedTriggers: " .. tostring(self.data)) end
			SellPriceTrigger.settings.sharedTriggers = self.data
			if #g_farmManager.farms > 1 then
				if self.data then
					for _,trigger in pairs(SellPriceTrigger.triggers) do
						trigger.farms = {}
					end
					SellPriceTrigger:addFarmTriggers(1)
					SellPriceTrigger:checkPrices(true)
				else
					SellPriceTrigger.triggers = {}
					SellPriceTrigger:fillTypes()
					SellPriceTrigger:checkPrices(true)
				end
			end
			
		end

		if self.id == SellPriceTrigger.EVENTS_ID.PREMISSIONS_SETTINGS then
			if SellPriceTrigger.debug then print("Change everyonePermissions: " .. tostring(self.data)) end
			SellPriceTrigger.settings.everyonePermission = self.data
		end

	end

	if g_server ~= nil then
		g_server:broadcastEvent(self, false)
		return
	end
end
