--
-- LoadSellPriceTriggerEvent
--
-- @Author: Tunis
-- @Date: 12.04.2022
-- @Version: 1.2.0.1
--
--
-- Changelog:
-- 1.2.0.0
-- Prices per farm support in MP
--

LoadSellPriceTriggerEvent = {}
LoadSellPriceTriggerEvent_mt = Class(LoadSellPriceTriggerEvent, Event)
InitEventClass(LoadSellPriceTriggerEvent, "LoadSellPriceTriggerEvent")

function LoadSellPriceTriggerEvent.emptyNew()
	local self = Event.new(LoadSellPriceTriggerEvent_mt)
	return self
end

function LoadSellPriceTriggerEvent.new(triggers, settings)
	if SellPriceTrigger.debug then print("LoadSellPriceTriggerEvent.new") end
	local self = LoadSellPriceTriggerEvent.emptyNew()
	self.triggers = triggers
	self.settings = settings
	return self
end

function LoadSellPriceTriggerEvent:readStream(streamId, connection)
	if SellPriceTrigger.debug then
		print("LoadSellPriceTriggerEvent:readStream")
	end
	if g_server == nil then
		self.settings = {}
		self.settings.sharedTriggers = streamReadBool(streamId)
		self.settings.everyonePermission = streamReadBool(streamId)
		local numTriggers = streamReadUInt8(streamId)
		local numFarms = streamReadUInt8(streamId)
		if SellPriceTrigger.debug then
			print("readStream sharedTriggers:" .. tostring(self.settings.sharedTriggers) .. " everyonePermission: " .. tostring(self.settings.everyonePermission))
			print("readStream numTriggers:" .. tostring(numTriggers) .. " numFarms: " .. tostring(numFarms))
		end
		self.triggers = {}
		for i = 1, numTriggers, 1 do
			local trigger = {}
			trigger.index = streamReadUInt8(streamId)
			trigger.farms = {}
			self.triggers[trigger.index] = trigger
			for j = 1, numFarms, 1 do
				local farm = {}
				farm.farmId = streamReadUInt8(streamId)
				farm.triggerPrice = streamReadUInt16(streamId)
				farm.lowest = streamReadFloat32(streamId)
				farm.highest = streamReadFloat32(streamId)
				farm.great = streamReadFloat32(streamId)
				farm.meanTotal = streamReadFloat32(streamId)
				farm.meanCount = streamReadUInt16(streamId)
				self.triggers[trigger.index].farms[farm.farmId] = farm
			end
		end
		
		self:run(connection)
	end
end

function LoadSellPriceTriggerEvent:writeStream(streamId, connection)

	if SellPriceTrigger.debug then
		print("LoadSellPriceTriggerEvent:writeStream")
	end

	streamWriteBool(streamId, self.settings.sharedTriggers)
	streamWriteBool(streamId, self.settings.everyonePermission)
	local triggerCount = 0
	local farmCount = 0
	for _,trigger in pairs(self.triggers) do
		triggerCount = triggerCount + 1
		if farmCount == 0 then
			for _ in pairs(trigger.farms) do
				farmCount = farmCount + 1
			end
		end
	end
	streamWriteUInt8(streamId, triggerCount)
	streamWriteUInt8(streamId, farmCount)

	for _,trigger in pairs(self.triggers) do
		if SellPriceTrigger.debug then print("writeStream index:" .. tostring(trigger.index)) end
		streamWriteUInt8(streamId, trigger.index)
		for _,farm in pairs(trigger.farms) do
			if SellPriceTrigger.debug then print("writeStream index:" .. tostring(farm.farmId)) end
			streamWriteUInt8(streamId, farm.farmId)
			streamWriteUInt16(streamId, farm.triggerPrice)
			streamWriteFloat32(streamId, farm.lowest)
			streamWriteFloat32(streamId, farm.highest)
			streamWriteFloat32(streamId, farm.great)
			streamWriteFloat32(streamId, farm.meanTotal)
			streamWriteUInt16(streamId, farm.meanCount)
		end
	end

end

function LoadSellPriceTriggerEvent:run(connection)
	if g_server ~= nil then
		g_server:broadcastEvent(LoadSellPriceTriggerEvent.new(SellPriceTrigger.triggers, SellPriceTrigger.settings), false)
		return
	end

	if(self.settings ~= nil) then
		SellPriceTrigger.settings = self.settings
	end

	if(self.triggers ~= nil) then
		SellPriceTrigger:fillTypes()
		if SellPriceTrigger.debug then print("L Trigger not nil") end
		for _,trigger in pairs(self.triggers) do
			for _,farm in pairs(trigger.farms) do
				SellPriceTrigger.triggers[trigger.index].farms[farm.farmId].triggerPrice = farm.triggerPrice
				SellPriceTrigger.triggers[trigger.index].farms[farm.farmId].lowest = farm.lowest
				SellPriceTrigger.triggers[trigger.index].farms[farm.farmId].mean = farm.meanTotal / farm.meanCount
				SellPriceTrigger.triggers[trigger.index].farms[farm.farmId].highest = farm.highest
				SellPriceTrigger.triggers[trigger.index].farms[farm.farmId].great = farm.great
				SellPriceTrigger.triggers[trigger.index].farms[farm.farmId].meanTotal = farm.meanTotal
				SellPriceTrigger.triggers[trigger.index].farms[farm.farmId].meanCount = farm.meanCount
			end
		end
	end
end
