--
-- SellPriceTriggerGUI
--
-- @Author: Tunis
-- @Date: 19.04.2022
-- @Version: 1.2.1.2
--
--
-- Changelog:
-- 1.2.1.1
-- Key assignmet for buttons
--
-- 1.2.0.0
-- Prices per farm support in MP
-- Moved settings to ingame menu
--
-- 1.1.0.0
-- MP support
-- +/- 50 buttons
--
-- 1.0.1.0
-- Current price and trend (climbing or falling) in table
--

SellPriceTriggerGUI = {}
SellPriceTriggerGUI.CONTROLS = {"sptTitle", "sptHeaderFilltype", "sptHeaderPrice", "sptHeaderGreatDemand", "sptHeaderLowest", "sptHeaderAverage", "sptHeaderHighest" , "sptHeaderTrigger", "sptTriggerList", "sptTriggerItemTemplate", "sptMenuButtons", "sptButtonReset", "sptButtonResetGreatDemand", "sptButtonDisable", "sptButtonHighest", "sptButtonAverage", "sptButtonHideShow", "sptMenuHelpBoxText", "sptMenuHelpBox"}
SellPriceTriggerGUI.color = {
	normal = {1,1,1,1}, --white
	over = {0.9,0.9,0.1,1}, --yellow
	up = {0.3763,0.6038,0.0782,1}, --green
	down = {0.8069,0.0097,0.0097,1}, --red
	great = {0.0742,0.4341,0.6939,1}, --blue
	transparent = {0,0,0,0}
}
SellPriceTriggerGUI.tableCols = {Image = 1, Filltype = 2, Trend = 3, Current = 4, Great = 5, Lowest = 6, Average = 7, Highest = 8, Trigger = 9}

local SellPriceTriggerGUI_mt = Class(SellPriceTriggerGUI, ScreenElement)

function SellPriceTriggerGUI:new(target)
	local obj = ScreenElement.new(target, SellPriceTriggerGUI_mt)
	obj:registerControls(SellPriceTriggerGUI.CONTROLS)
	obj.performBackgroundBlur = true
	return obj
end

function SellPriceTriggerGUI:onCreate()
	self.sptTriggerItemTemplate:unlinkElement()
	self.sptTriggerItemTemplate:setVisible(false)

	self.sptTitle.text = g_i18n:getText('SPT_gui_Title')

	self.sptHeaderFilltype.text = g_i18n:getText('SPT_gui_HeaderFilltype')
	self.sptHeaderPrice.text = g_i18n:getText('SPT_gui_HeaderPrice')
	self.sptHeaderGreatDemand.text = g_i18n:getText('SPT_gui_HeaderGreatDemand')
	self.sptHeaderLowest.text = g_i18n:getText('SPT_gui_HeaderLowest')
	self.sptHeaderAverage.text = g_i18n:getText('SPT_gui_HeaderAverage')
	self.sptHeaderHighest.text = g_i18n:getText('SPT_gui_HeaderHighest')
	self.sptHeaderTrigger.text = g_i18n:getText('SPT_gui_HeaderTrigger')

	self.sptButtonReset:setText(g_i18n:getText('SPT_gui_Reset'))
	self.sptButtonResetGreatDemand:setText(g_i18n:getText('SPT_gui_ResetGreatDemand'))
	self.sptButtonDisable:setText(g_i18n:getText('SPT_gui_Disable'))
	self.sptButtonHighest:setText(g_i18n:getText('SPT_gui_Highest'))
	self.sptButtonAverage:setText(g_i18n:getText('SPT_gui_Average'))
	self.sptButtonHideShow:setText(g_i18n:getText('SPT_gui_Hide'))

	self.isHideDisabled = false

end

function SellPriceTriggerGUI:onOpen()
	if SellPriceTrigger.settings.sharedTriggers then
		self.farmId = 1
	else
		self.farmId = g_currentMission.player.farmId
	end
	if SellPriceTrigger.settings.everyonePermission then
		self.isFarmManager = true
	else
		self.isFarmManager = g_farmManager.farmIdToFarm[self.farmId].userIdToPlayer[g_currentMission.player.userId].isFarmManager
	end

	self:loadList(self.farmId, self.isHideDisabled)
	self.updateTimer = 0.0

	if not self.isFarmManager then
		self.sptButtonReset:setDisabled(true)
		self.sptButtonResetGreatDemand:setDisabled(true)
		self.sptButtonDisable:setDisabled(true)
		self.sptButtonHighest:setDisabled(true)
		self.sptButtonAverage:setDisabled(true)
		for _,element in pairs(self.sptMenuButtons.elements[4].elements) do
			element:setDisabled(true)
		end
		for _,element in pairs(self.sptMenuButtons.elements[5].elements) do
			element:setDisabled(true)
		end
	elseif self.sptButtonReset.disabled then
		self.sptButtonReset:setDisabled(false)
		self.sptButtonResetGreatDemand:setDisabled(false)
		self.sptButtonDisable:setDisabled(false)
		self.sptButtonHighest:setDisabled(false)
		self.sptButtonAverage:setDisabled(false)
		for _,element in pairs(self.sptMenuButtons.elements[4].elements) do
			element:setDisabled(false)
		end
		for _,element in pairs(self.sptMenuButtons.elements[5].elements) do
			element:setDisabled(false)
		end
	end

	if self.isFarmManager then
		self:setInputAction()
	end

	SellPriceTriggerGUI:superClass().onOpen(self)
	return false
end

function SellPriceTriggerGUI:onClose()
	SellPriceTriggerGUI:superClass().onClose(self)
	g_gui.inputManager:removeActionEventsByTarget(self)
end

function SellPriceTriggerGUI:update(dt)
	SellPriceTriggerGUI:superClass().update(self, dt)
	self.updateTimer = self.updateTimer + dt * g_currentMission.missionInfo.timeScale
	if self.updateTimer >= SellPriceTrigger.updateRun then
		self:changeList(self.farmId)
	end
end

function SellPriceTriggerGUI:onClickBack()
	self:changeScreen(nil)
	SellPriceTriggerGUI:superClass().onClickBack(self)
end

function SellPriceTriggerGUI:onClickOk()
	SellPriceTriggerGUI:superClass().onClickOk(self)
	if self.isFarmManager then
		self:saveList(self.farmId)
	end
	self:onClickBack()
end

function SellPriceTriggerGUI:onSptMenuHelpTextChanged(element)
	if element.text == "" then
		self.sptMenuHelpBox:setVisible(false)
	else
		self.sptMenuHelpBox:setVisible(true)
	end
end

function SellPriceTriggerGUI:onHighlightReset()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_ResetToolTip'))
end

function SellPriceTriggerGUI:onHighlightResetGreatDemand()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_ResetGreatDemandToolTip'))
end

function SellPriceTriggerGUI:onHighlightDisable()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_DisableToolTip'))
end

function SellPriceTriggerGUI:onHighlightHighest()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_HighestToolTip'))
end

function SellPriceTriggerGUI:onHighlightAverage()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_AverageToolTip'))
end

function SellPriceTriggerGUI:onHighlightUp()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_UpToolTip'))
end

function SellPriceTriggerGUI:onHighlightDown()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_DownToolTip'))
end

function SellPriceTriggerGUI:onHighlightHideShow()
	self.sptMenuHelpBoxText:setText(g_i18n:getText('SPT_gui_HideShowToolTip'))
end

function SellPriceTriggerGUI:onHighlightRemove()
	self.sptMenuHelpBoxText:setText('')
end

function SellPriceTriggerGUI:onClickReset()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		SellPriceTrigger.triggers[element.trigger.index].farms[self.farmId].highest = 0.0
		SellPriceTrigger.triggers[element.trigger.index].farms[self.farmId].lowest = 0.0
		SellPriceTrigger.triggers[element.trigger.index].farms[self.farmId].mean = 0.0
		SellPriceTrigger.triggers[element.trigger.index].farms[self.farmId].meanTotal = 0.0
		SellPriceTrigger.triggers[element.trigger.index].farms[self.farmId].meanCount = 0
		SellPriceTrigger:checkPrices()
		SellPriceTrigger:meanPrice(element.trigger.index, self.farmId)
		element.trigger.lowest = SellPriceTrigger.triggers[element.trigger.index].farms[self.farmId].lowest
		element.trigger.mean = SellPriceTrigger.triggers[element.trigger.index].farms[self.farmId].mean
		element.trigger.highest = SellPriceTrigger.triggers[element.trigger.index].farms[self.farmId].highest
		element.elements[self.tableCols.Lowest]:setText(g_i18n:formatMoney(element.trigger.lowest))
		element.elements[self.tableCols.Average]:setText(g_i18n:formatMoney(element.trigger.mean))
		element.elements[self.tableCols.Highest]:setText(g_i18n:formatMoney(element.trigger.highest))
		SellPriceTrigger:onChangeEvent(SellPriceTrigger.EVENTS_ID.RESET, self.farmId, SellPriceTrigger.triggers[element.trigger.index])
	end
end

function SellPriceTriggerGUI:onClickResetGreatDemand()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		SellPriceTrigger.triggers[element.trigger.index].farms[self.farmId].great = 0.0
		SellPriceTrigger:checkPrices()
		element.trigger.great = SellPriceTrigger.triggers[element.trigger.index].farms[self.farmId].great
		if element.trigger.great > 0 then
			element.elements[self.tableCols.Great]:setText(g_i18n:formatMoney(element.trigger.great))
		else
			element.elements[self.tableCols.Great]:setText('-')
		end
		SellPriceTrigger:onChangeEvent(SellPriceTrigger.EVENTS_ID.RESET_GREAT, self.farmId, SellPriceTrigger.triggers[element.trigger.index])
	end
end

function SellPriceTriggerGUI:onClickDisable()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		element.trigger.price = 0
		element.elements[self.tableCols.Trigger]:setText(g_i18n:getText('SPT_gui_Disabled'))
		self:priceOver(element)
	end
end

function SellPriceTriggerGUI:onClickHighest()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil and element.trigger.highest > 0 then
		element.trigger.price = math.floor(element.trigger.highest)
		element.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(element.trigger.price))
		self:priceOver(element)
	end
end

function SellPriceTriggerGUI:onClickAverage()
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil and element.trigger.mean > 0 then
		element.trigger.price = math.floor(element.trigger.mean)
		element.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(element.trigger.price))
		self:priceOver(element)
	end
end

function SellPriceTriggerGUI:onClickUp(number)
	if number == nil or type(number) ~= "number" then number = 1 end
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		local price = element.trigger.price
		price = price + number
		local trigger = element.trigger
		element.trigger.price = price
		element.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(element.trigger.price))
		self:priceOver(element)
	end
end

function SellPriceTriggerGUI:onClickUpTen()
	self:onClickUp(10)
end

function SellPriceTriggerGUI:onClickUpFifty()
	self:onClickUp(50)
end

function SellPriceTriggerGUI:onClickDown(number)
	if number == nil or type(number) ~= "number" then number = 1 end
	local element = self.sptTriggerList:getSelectedElement()
	if self.sptTriggerList:getItemCount() > 0 and element ~= nil then
		local price = element.trigger.price
		if price > 0 then
			price = price - number
			if price < 0 then price = 0 end
			element.trigger.price = price
			if price == 0 then
				element.elements[self.tableCols.Trigger]:setText(g_i18n:getText('SPT_gui_Disabled'))
			else
				element.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(element.trigger.price))
			end
			self:priceOver(element)
		end
	end
end

function SellPriceTriggerGUI:onClickDownTen()
	self:onClickDown(10)
end

function SellPriceTriggerGUI:onClickDownFifty()
	self:onClickDown(50)
end

function SellPriceTriggerGUI:onClickHideShow()
	if self.isHideDisabled then
		self.sptButtonHideShow:setText(g_i18n:getText('SPT_gui_Hide'))
		self.isHideDisabled = false
	else
		self.sptButtonHideShow:setText(g_i18n:getText('SPT_gui_Show'))
		self.isHideDisabled = true
	end	
	self:loadList(self.farmId,self.isHideDisabled)
end

function SellPriceTriggerGUI:loadList(farmId, hideDisabled)

	self.sptTriggerList:deleteListItems()

	for _,trigger in pairs(SellPriceTrigger.triggers) do
		if trigger.farms[farmId].highest > 0 and not(hideDisabled and trigger.farms[farmId].triggerPrice == 0)then -- show only filltypes that can be sold
			local new = self.sptTriggerItemTemplate:clone(self.sptTriggerList)
			new:setVisible(true)
			local filltype = g_fillTypeManager.fillTypes[trigger.index]
			new.elements[self.tableCols.Image]:setImageFilename(filltype.hudOverlayFilename)
			new.elements[self.tableCols.Filltype]:setText(filltype.title)
			new.elements[self.tableCols.Current]:setText(g_i18n:formatMoney(trigger.currentPrice))
			if trigger.farms[farmId].great > 0 then
				new.elements[self.tableCols.Great]:setText(g_i18n:formatMoney(trigger.farms[farmId].great))
			else
				new.elements[self.tableCols.Great]:setText('-')
			end
			new.elements[self.tableCols.Lowest]:setText(g_i18n:formatMoney(trigger.farms[farmId].lowest))
			new.elements[self.tableCols.Average]:setText(g_i18n:formatMoney(trigger.farms[farmId].mean))
			new.elements[self.tableCols.Highest]:setText(g_i18n:formatMoney(trigger.farms[farmId].highest))
			if trigger.farms[farmId].triggerPrice == 0 then
				new.elements[self.tableCols.Trigger]:setText(g_i18n:getText('SPT_gui_Disabled'))
			else
				new.elements[self.tableCols.Trigger]:setText(g_i18n:formatMoney(trigger.farms[farmId].triggerPrice))
			end
			new:updateAbsolutePosition()
			new.trigger = {}
			new.trigger['index'] = trigger.index
			new.trigger['price'] = trigger.farms[farmId].triggerPrice
			new.trigger['highest'] = trigger.farms[farmId].highest
			new.trigger['great'] = trigger.farms[farmId].great
			new.trigger['lowest'] = trigger.farms[farmId].lowest
			new.trigger['mean'] = trigger.farms[farmId].mean

			self:priceOver(new)
			self:priceTrendIcon(new)
		end
	end

end

function SellPriceTriggerGUI:saveList(farmId)
	local triggers = {}
	for _,element in pairs(self.sptTriggerList.elements) do
		if element.trigger ~= nil then
			local trigger = {}
			trigger.index = element.trigger.index
			trigger.price = element.trigger.price
			if SellPriceTrigger.debug then
				print("---ListSave "..tostring(element.trigger.index).." price "..tostring(element.trigger.price))
			end
			triggers[element.trigger.index] = trigger
		end
	end
	SellPriceTrigger:onChangeEvent(SellPriceTrigger.EVENTS_ID.SAVE_TRIGGERS, farmId, triggers)
end

function SellPriceTriggerGUI:changeList(farmId)
	for _,element in pairs(self.sptTriggerList.elements) do
		if element.trigger ~= nil then
			element.trigger.lowest = SellPriceTrigger.triggers[element.trigger.index].farms[farmId].lowest
			element.trigger.mean = SellPriceTrigger.triggers[element.trigger.index].farms[farmId].mean
			element.trigger.highest = SellPriceTrigger.triggers[element.trigger.index].farms[farmId].highest
			element.elements[self.tableCols.Current]:setText(g_i18n:formatMoney(SellPriceTrigger.triggers[element.trigger.index].currentPrice))
			element.elements[self.tableCols.Lowest]:setText(g_i18n:formatMoney(element.trigger.lowest))
			element.elements[self.tableCols.Average]:setText(g_i18n:formatMoney(element.trigger.mean))
			element.elements[self.tableCols.Highest]:setText(g_i18n:formatMoney(element.trigger.highest))
			element.trigger.great = SellPriceTrigger.triggers[element.trigger.index].farms[farmId].great
			if element.trigger.great > 0 then
				element.elements[self.tableCols.Great]:setText(g_i18n:formatMoney(element.trigger.great))
			else
				element.elements[self.tableCols.Great]:setText('-')
			end
			self:priceOver(element)
			self:priceTrendIcon(element)
		end
	end
end

function SellPriceTriggerGUI:priceOver(element)
	if(SellPriceTrigger.triggers[element.trigger.index].currentPrice > element.trigger.price and element.trigger.price > 0) then
		element.elements[self.tableCols.Trigger].textColor = self.color.over
		element.elements[self.tableCols.Trigger].textSelectedColor = self.color.over
		element.elements[self.tableCols.Trigger].textHighlightedColor = self.color.over
	else
		element.elements[self.tableCols.Trigger].textColor = self.color.normal
		element.elements[self.tableCols.Trigger].textSelectedColor = self.color.normal
		element.elements[self.tableCols.Trigger].textHighlightedColor = self.color.normal
	end
end

function SellPriceTriggerGUI:priceTrendIcon(element)
	local PRICE_CLIMBING = 4
	local PRICE_FALLING = 2
	local PRICE_GREAT_DEMAND = 5

	if SellPriceTrigger.triggers[element.trigger.index].currentTrend == PRICE_FALLING then
		element.elements[self.tableCols.Trend].elements[1]:setImageColor(nil,self.color.down[1],self.color.down[2],self.color.down[3],self.color.down[4])
		element.elements[self.tableCols.Trend].elements[1]:setImageRotation(3.1416)
	elseif SellPriceTrigger.triggers[element.trigger.index].currentTrend == PRICE_CLIMBING then
		element.elements[self.tableCols.Trend].elements[1]:setImageColor(nil,self.color.up[1],self.color.up[2],self.color.up[3],self.color.up[4])
		element.elements[self.tableCols.Trend].elements[1]:setImageRotation(0)
	elseif SellPriceTrigger.triggers[element.trigger.index].currentTrend == PRICE_GREAT_DEMAND then
		element.elements[self.tableCols.Trend].elements[1]:setImageColor(nil,self.color.great[1],self.color.great[2],self.color.great[3],self.color.great[4])
		element.elements[self.tableCols.Trend].elements[1]:setImageRotation(1.5708)
	else
		element.elements[self.tableCols.Trend].elements[1]:setImageColor(nil,self.color.transparent[1],self.color.transparent[2],self.color.transparent[3],self.color.transparent[4])
	end
end

function SellPriceTriggerGUI:setInputAction()
	g_gui.inputManager:registerActionEvent(InputAction.SPT_DISABLE, self, SellPriceTriggerGUI.onClickDisable, false, true, false, true, nil, nil, true)
	g_gui.inputManager:registerActionEvent(InputAction.SPT_HIGHEST, self, SellPriceTriggerGUI.onClickHighest, false, true, false, true, nil, nil, true)
	g_gui.inputManager:registerActionEvent(InputAction.SPT_AVERAGE, self, SellPriceTriggerGUI.onClickAverage, false, true, false, true, nil, nil, true)
	g_gui.inputManager:registerActionEvent(InputAction.SPT_PLUS, self, SellPriceTriggerGUI.onClickUp, false, true, false, true, nil, nil, true)
	g_gui.inputManager:registerActionEvent(InputAction.SPT_PLUS_10, self, SellPriceTriggerGUI.onClickUpTen, false, true, false, true, nil, nil, true)
	g_gui.inputManager:registerActionEvent(InputAction.SPT_PLUS_50, self, SellPriceTriggerGUI.onClickUpFifty, false, true, false, true, nil, nil, true)
	g_gui.inputManager:registerActionEvent(InputAction.SPT_MINUS, self, SellPriceTriggerGUI.onClickDown, false, true, false, true, nil, nil, true)
	g_gui.inputManager:registerActionEvent(InputAction.SPT_MINUS_10, self, SellPriceTriggerGUI.onClickDownTen, false, true, false, true, nil, nil, true)
	g_gui.inputManager:registerActionEvent(InputAction.SPT_MINUS_50, self, SellPriceTriggerGUI.onClickDownFifty, false, true, false, true, nil, nil, true)
	g_gui.inputManager:registerActionEvent(InputAction.SPT_RESET, self, SellPriceTriggerGUI.onClickReset, false, true, false, true, nil, nil, true)
	g_gui.inputManager:registerActionEvent(InputAction.SPT_RESET_GREAT, self, SellPriceTriggerGUI.onClickResetGreatDemand, false, true, false, true, nil, nil, true)
	g_gui.inputManager:registerActionEvent(InputAction.SPT_HIDE_SHOW, self, SellPriceTriggerGUI.onClickHideShow, false, true, false, true, nil, nil, true)
end
